<?php

/**
 * @package     Joomla.Plugin
 * @subpackage  Task.resourcespace
 *
 * @copyright   (C) 2023 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Joomla\Plugin\Task\ResourceSpace\Extension;

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\Http\Exception\HttpException;
use Joomla\CMS\Http\Http;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Version;
use Joomla\Component\Scheduler\Administrator\Event\ExecuteTaskEvent;
use Joomla\Component\Scheduler\Administrator\Task\Status;
use Joomla\Component\Scheduler\Administrator\Traits\TaskPluginTrait;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Event\SubscriberInterface;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * A task plugin. Offers 2 task routines Invalidate Expired Consents and Remind Expired Consents
 * {@see ExecuteTaskEvent}.
 *
 * @since 5.0.0
 */
final class ResourceSpace extends CMSPlugin implements SubscriberInterface
{
    use DatabaseAwareTrait;
    use TaskPluginTrait;

    /**
     * @var string[]
     * @since 5.0.0
     */
    private const TASKS_MAP = [
        'update.notification' => [
            'langConstPrefix' => 'RESOURCESPACE',
            'method'          => 'sendNotification',
            'form'            => 'sendForm',
        ],
    ];

    /**
     * @var boolean
     * @since 5.0.0
     */
    protected $autoloadLanguage = true;

    /**
     * @inheritDoc
     *
     * @return string[]
     *
     * @since 5.0.0
     */
    public static function getSubscribedEvents(): array
    {
        return [
            'onTaskOptionsList'    => 'advertiseRoutines',
            'onExecuteTask'        => 'standardRoutineHandler',
            'onContentPrepareForm' => 'enhanceTaskItemForm',
        ];
    }

    /**
     * Method to send the update notification.
     *
     * @param   ExecuteTaskEvent  $event  The `onExecuteTask` event.
     *
     * @return integer  The routine exit code.
     *
     * @since  5.0.0
     * @throws \Exception
     */
    private function sendNotification(ExecuteTaskEvent $event): int
    {
        // Load the parameters.
        $this->logTask('ResourceSpace Starting task');
        $specificEmail = $event->getArgument('params')->email ?? '';

        $helperPath = JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components/com_resourcespace' . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR . 'resourcespace.php';

        // Check if the helper file exists and include it
        if (file_exists($helperPath)) {
            $this->logTask('ResourceSpace path exist');
            require_once $helperPath;
            try {
                $this->logTask('Executing');
                $res = self::fetchForCronJob();

                $this->logTask('Executed' . count($res));
            } catch (\Throwable $th) {
                $this->logTask('Error: ' . $th->getMessage());
            }
            // Example method
        } else {

        }

        // Get the update model and retrieve the Joomla! core updates
        // $model = $this->getApplication()->bootComponent('com_installer')
        //     ->getMVCFactory()->createModel('Update', 'Administrator', ['ignore_request' => true]);

        // if (false) {
        //     return Status::KNOCKOUT;
        // }

        $this->logTask('ResourceSpace end');

        return Status::OK;
    }

    private function getConfig()
    {
        // Get component params
        $params = ComponentHelper::getParams('com_resourcespace');

        // Retrieve stored values
        $apiUrl   = trim($params->get('resourcespace_api_url', ''));
        $username = trim($params->get('resourcespace_username', ''));
        $apiKey   = trim($params->get('resourcespace_api_key', ''));

        // Validate if all required values are set
        if (empty($apiUrl) || empty($username) || empty($apiKey)) {
            return ['error' => 'No API key added. Please configure the ResourceSpace settings.'];
        }

        return [
            'apiUrl'   => $apiUrl,
            'username' => $username,
            'apiKey'   => $apiKey,
        ];
    }

    /**
     * Makes an API request to the given URL using Joomla's HTTP client.
     *
     * @param   string  $url  The API endpoint URL.
     *
     * @return  mixed   The API response body or null in case of failure.
     */
    private function makeApiRequest($url)
    {
        try {
            $httpClient = new Http();
            $response   = $httpClient->get($url, [
                'timeout' => 40,
                'verify'  => false,
            ]);
            return (string) $response->getBody();
        } catch (HttpException $e) {
            Factory::getApplication()->enqueueMessage(Text::sprintf('API request failed: %s', $e->getMessage()), 'error');
            return null;
        }
    }

    private function getResourcesCron()
    {
        // Load configuration.
        $config   = self::getConfig();
        $apiUrl   = $config['apiUrl'];
        $username = $config['username'];
        $apiKey   = $config['apiKey'];

        if (! $apiUrl || ! $username || ! $apiKey) {
            return [];
        }

        $searchTerm = "";

        $query = "user={$username}&function=do_search&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);
        $fullUrl   = "{$apiUrl}?{$query}&sign={$signature}";

        // Make the API request.
        $response = self::makeApiRequest($fullUrl);
        return $response;
    }
    private function fetchForCronJob()
    {
        // Get the API configuration
        $config = self::getConfig();

        // Check for an error in configuration
        if (isset($config['error'])) {
            return $config['error'];
        }

        $response = self::getResourcesCron();
        $data     = json_decode($response, true);
        $results  = $data;
        $ref_ids  = [];
        if ($response && is_array($data)) {
            foreach ($data as $resource) {
                $imageUrl      = self::getResoucePath($resource['ref'], $resource['file_extension']);
                $last_modified = $resource['file_modified'];
                if ($imageUrl != null) {
                    self::saveImageFromUrl($imageUrl, $resource['field8'], $resource['ref'], $resource['file_extension']);
                }
                $ref_ids[] = $resource['ref'];
            }
        }

        // Check for errors
        if (! $response) {
            return 'API request failed. Please check your API URL and credentials.';
        }

        if (! empty($ref_ids)) {
            self::deleteMissingMedia($ref_ids);
        }

        return $results;
    }

    private function getResoucePath($ref_id, $extension)
    {

        // Load configuration.
        $config   = self::getConfig();
        $apiUrl   = $config['apiUrl'];
        $username = $config['username'];
        $apiKey   = $config['apiKey'];

        if (! $apiUrl || ! $username || ! $apiKey) {
            return [];
        }

        $query = "user={$username}&function=get_resource_path&external=true&extension={$extension}&ref=" . urlencode($ref_id);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        // Make the API request.
        $response = self::makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);

        return $resources;
    }

    private function deleteMissingMedia($externalRefIds)
    {
        // Get the database connection
        $db = Factory::getDbo();

        // Get all media `ref_id`s from the database
        $query = $db->getQuery(true)
            ->select($db->quoteName(['id', 'ref_id', 'media_id', 'image_path']))
            ->from($db->quoteName('#__resourcespace_media'));
        $db->setQuery($query);
        $allMedia = $db->loadObjectList();

        $mediaToDelete  = [];
        $assetsToDelete = [];
        $filesToDelete  = [];

        foreach ($allMedia as $media) {
            if (! in_array($media->ref_id, $externalRefIds)) {
                $mediaToDelete[]  = $media->id;                      // Collect media IDs for deletion
                $assetsToDelete[] = $media->media_id;                // Collect asset IDs for deletion
                $filesToDelete[]  = JPATH_ROOT . $media->image_path; // Collect file paths
            }
        }

        // 🔹 Delete from `#__resourcespace_media`
        if (! empty($mediaToDelete)) {
            $query = $db->getQuery(true)
                ->delete($db->quoteName('#__resourcespace_media'))
                ->where($db->quoteName('id') . ' IN (' . implode(',', $mediaToDelete) . ')');
            $db->setQuery($query);
            $db->execute();
        }

        // 🔹 Delete from `#__assets`
        if (! empty($assetsToDelete)) {
            $query = $db->getQuery(true)
                ->delete($db->quoteName('#__assets'))
                ->where($db->quoteName('id') . ' IN (' . implode(',', $assetsToDelete) . ')');
            $db->setQuery($query);
            $db->execute();
        }

        // 🔹 Delete the images from disk
        foreach ($filesToDelete as $filePath) {
            if (file_exists($filePath)) {
                unlink($filePath);
            }
        }
    }

    /**
     * Downloads and saves an image from a URL to Joomla's media folder and adds metadata.
     *
     * @param string $imageUrl The URL of the image to download.
     * @param string $imageName The name to save the image.
     * @param string $refId The reference ID to associate with the image.
     * @param string $extension The extension of the image file.
     *
     * @return string|false The path to the saved image, or false if there was an error.
     */

    private function saveImageFromUrl($imageUrl, $imageName, $refId, $extension)
    {
        $mediaPath = JPATH_ROOT . '/images/resourcespace/';

        // Ensure the media folder exists
        if (! Folder::exists($mediaPath)) {
            Folder::create($mediaPath);
        }

        // Full path to save the image
        $savePath = $mediaPath . $imageName . '.' . $extension;

        // Download the image using Joomla's HTTP class
        $http     = HttpFactory::getHttp();
        $response = $http->get($imageUrl);

        // Check if the request was successful
        if ($response->code == 200) {
            // Save the image content to the media folder
            file_put_contents($savePath, $response->body);

            // Now we insert the metadata into Joomla's media table
            $db = Factory::getDbo();

            // Check if the media already exists in resourcespace_media table
            $query = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__resourcespace_media'))
                ->where($db->quoteName('ref_id') . ' = ' . $db->quote($refId));
            $db->setQuery($query);
            $existingMedia = $db->loadObject();

            // Check if the asset already exists
            $query2 = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__assets'))
                ->where($db->quoteName('title') . ' = ' . $db->quote($refId));
            $db->setQuery($query2);
            $existingAsset = $db->loadObject();

            // Prepare the asset metadata
            $asset            = new \stdClass();
            $asset->parent_id = 1; // Typically, 1 is the root asset
            $asset->title     = $refId;
            $asset->name      = OutputFilter::stringURLSafe(substr($imageName, 0, 50));
            $asset->path      = '/images/resourcespace/' . substr($imageName, 0, 95) . '.' . $extension; // Path to the saved image
            $asset->ref_id    = $refId;                                                                  // Custom meta field
            $asset->published = 1;                                                                       // Make the asset published
            $asset->level     = 1;
            $asset->rules     = '{}';

            if ($existingAsset) {
                                                 // Update the existing asset
                $asset->id = $existingAsset->id; // Set the ID to update the existing record
                $db->updateObject('#__assets', $asset, 'id');
            } else {
                // Insert a new asset
                $db->insertObject('#__assets', $asset);
                $asset->id = $db->insertid();
            }

            // Prepare the media metadata
            $mediaData              = new \stdClass();
            $mediaData->ref_id      = $refId;
            $mediaData->image_name  = $imageName;
            $mediaData->image_path  = '/images/resourcespace/' . $imageName . '.' . $extension;
            $mediaData->media_id    = $existingAsset ? $existingAsset->id : $asset->id; // Link to assets table
            $mediaData->modified_at = date('Y-m-d H:i:s');                              // Update timestamp

            if ($existingMedia) {
                $mediaData->id = $existingMedia->id;
                $db->updateObject('#__resourcespace_media', $mediaData, 'id');
            } else {
                $mediaData->created_at = date('Y-m-d H:i:s'); // Set created timestamp
                $db->insertObject('#__resourcespace_media', $mediaData);
            }

            // Return the path to the saved image
            return '/images/resourcespace/' . $imageName . '.' . $extension;
        } else {
            // Return false if the image could not be downloaded
            return false;
        }
    }

}
